;; -*- mode: a51; mode: fold -*-
;; Kleine Bibliothek zur Ein/Ausgabe ber das minicom ;{{{
;; 
;; write_char
;;   Eingang:
;;     Zeichen in a
;;   Wirkung:
;;     schreibt Zeichen in a in das Terminalfenster
;;   Ausgang:
;;     keiner
;; 
;; write_string
;;   Eingang: 
;;     Anfangsadresse eines Nullterminierten Strings in dptr
;;   Wirkung:
;;     schreibt String in das Terminalfenster
;;     zerstrt a
;;   Ausgang:
;;     keiner
;;
;; write_decimal8
;;   Eingang
;;     8-Bit Integerwert in a
;;   Wirkung:
;;     Gibt Integerzahl als Dezimalzahl im Terminalfenster aus.
;;     zerstrt dptr
;;   Ausgang
;;     keiner
;;
;; clrscr
;;   Eingang:
;;     keiner
;;   Wirkung:
;;     lscht den Bildschirm des Terminals
;;
;; go_left
;;   Eingang:
;;     keiner
;;   Wirkung:
;;     Schreibmarke im Terminalfenster wird um eine Position nach links
;;     versetzt
;;
;; read_char
;;   Eingang:
;;     keiner
;;   Wirkung:
;;     liest Zeichen von der Tastatur
;;   Ausgang:
;;     gelesenes Zeichen in a
;;
;; read_string
;;   Eingang:
;;     16-Bit Anfangsadresse eines Buffers <buf> von <n> Zeichen in dptr
;;     Lnge <n> des Buffers in r7
;;   Wirkung:
;;     Liest String von der Tastatatur in <buf> ein. String wird  la
;;     C mit einem Null-Zeichen abgeschlossen
;;     (Einfachster Zeileneditor, beherrscht nur Backspace)
;;     zerstrt r6
;;   Ausgang:
;;     Stringlnge in a
;;
;; strlen
;;   Eingang:
;;     16-Bit Anfangsadresse eines Nullterminierten Strings
;;   Wirkung:
;;     Berechnet die Lnge des String
;;     Zerstrt a und r7
;;   Ausgang:
;;     Stringlnge in a
;;
;; sleep
;;   Eingang:
;;     keiner
;;   Wirkung:
;;     wartet ca. 1 Sekunde
;;     zerstrt r5, r6 und r7
;;   Ausgang:
;;     keiner
;;
;; gotoxy
;;   Eingang:
;;     Zeilen- und Spaltenindex in r6 und r7
;;   Wirkung:
;;     Setzt den Cursor auf die angegebene Position
;;   Ausgang:
;;     keiner
;;
;; Bernd Nawothnig 5/2001
;;
;}}}
.equ	SEED,		0x62
.equ	LF,		10
.equ	CR,		13
.equ 	ESC,		27
.equ	BACKSPACE,	8
.equ	MONITOR,	0x918e
.equ	buffsize,	8

.org 	0x0100

;; kleines Testprogramm ...
	clr	ES
	setb	TI
	acall	clrscr
 funny:
 	mov	a, #15
 	acall	rnd
 	inc	a
 	mov	r6, a
 	mov	a, #60
 	acall	rnd
 	inc	a
 	mov	r7, a
 	acall	gotoxy
 	mov	a,#26
 	acall	rnd
 	add	a,#'A'
 	acall	write_char
 	sjmp	funny
;funny:
; 	acall	rand
; 	mov	r6, #2
; 	mov	r7, #15
; 	acall	gotoxy
; 	mov	dptr, #buffer
; 	mov	r7, #buffsize
; 	acall	read_string
; 	mov	dptr, #buffer
; 	acall	write_string
; 	acall	write_space
; 	mov	dptr, #buffer
; 	acall	strlen
; 	acall	write_decimal8
; 	acall	newline
	
	setb	ES
	ljmp	MONITOR

buffer:
.skip	buffsize

;; Bibliothek
dec_dptr: ;{{{
; dptr dekrementieren ohne Akku zu zerstren
	push	ACC
	mov	a, DPL 
	clr	c
	subb	a, #1
	mov	DPL, a
	mov	a, DPH
	subb	a, #0
	mov	DPH, a
	pop	ACC
	ret
;}}}
read_string: ;{{{
; simpler Zeileneditor
	mov	a, r7	
	mov	r6, a

loop:
	acall	read_char	
	cjne	a, #BACKSPACE, test_escape
	mov	a, r7
	clr	c
	subb	a, r6 ; prfen, ob man bereits am Zeilenanfang ist
	jz	loop
	mov	a, #BACKSPACE
	acall	write_char	
	acall	dec_dptr
	inc	r7
	
	mov	a, #' '
	acall	write_char
	acall	go_left
	sjmp	loop
test_escape:	
  	cjne	a, #ESC, test_enter
  	mov	a, #'\\'
 	acall	put_char
  	mov	a, #'e'
continue:	
	acall	put_char
	sjmp	loop
	
test_enter:
	cjne	a, #CR, continue

finis:
	acall	write_char	; CR/LF an das Terminal senden
	mov	a,#LF
	acall	write_char
	clr	a
	movx	@dptr,a		; String mit 0 terminieren
	mov	a, r6		
	clr	c
	subb	a, r7		; Stringlnge berechnen
	ret
;}}}
put_char: ;{{{
; Byte an String anhngen und ausgeben
	cjne	r7, #1, pb1
	sjmp	return
pb1:	movx	@dptr, a
	inc	dptr
	dec	r7
;}}}
write_char: ;{{{
; muss direkt nach put_char stehen!
w4:	jnb	TI, w4
	clr	TI
	mov	SBUF, a
	ret
;}}}
write_space: ;{{{
	mov	a, #' '
	sjmp	write_char
;}}}
read_char: ;{{{
	jnb	RI, read_char
	clr	RI
	mov	a, SBUF
	ret
;}}}
go_left: ;{{{
	mov	a, #ESC
	acall	write_char
	mov	a, #'['
	acall	write_char
	mov	a, #'D'
	acall	write_char
	ret
;}}}
go_right: ;{{{
	mov	a, #ESC
	acall	write_char
	mov	a, #'['
	acall	write_char
	mov	a, #'C'
	acall	write_char
	ret
;}}}
go_up: ;{{{
	mov	a, #ESC
	acall	write_char
	mov	a, #'['
	acall	write_char
	mov	a, #'A'
	acall	write_char
	ret
;}}}
go_down: ;{{{
	mov	a, #ESC
	acall	write_char
	mov	a, #'['
	acall	write_char
	mov	a, #'B'
	acall	write_char
	ret
;}}}
write_decimal8: ;{{{
	mov	dptr,#buff_decimal+3
wd0:	
	acall	dec_dptr
	mov	B,#10
	div	ab
	mov	r6,a
	mov	a,B
	add	a,#'0'
	movx	@dptr,a
	mov	a,r6
	jnz	wd0	
;}}}
write_string: ;{{{
; write_string muss unmittelbar hinter write_decimal8 stehen!
	movx	a, @dptr
	jz	return
	acall	write_char
	inc	dptr
	sjmp	write_string
;}}}
strlen: ;{{{
	mov	r7, #-1
str1:	inc	r7
	movx	a,@dptr
	inc	dptr
	jnz	str1
	mov	a, r7
	ret
	
;}}}
clrscr: ;{{{
	push	DPL
	push	DPH
	mov	dptr, #CLR_SCR
	acall	write_string
	pop	DPH
	pop	DPL
return:	ret

;}}}
newline: ;{{{
	mov	a, #CR
	lcall	write_char
	mov	a, #LF
	ljmp	write_char
;}}}
sleep: ;{{{
	mov	r7, #15
	mov	r6, #0
	mov	r5, #0
wait0:	djnz	r5, wait0
	djnz	r6, wait0
	djnz	r7, wait0
	ret
;}}}
gotoxy: ;{{{
	mov	a, #ESC
	acall	write_char
	mov	a, #'['
	acall	write_char
	mov	a, r6
	acall	write_decimal8
	mov	a, #';'
	acall	write_char
	mov	a, r7
	acall	write_decimal8
	mov	a, #'H'
	ljmp	write_char
;}}}
rand: ;{{{
	mov	a, SEED+1
	mov	b, #0xe5	; Low-Byte des Multiplikators 0x6fe5
	mul	ab		; Low-Byte des Ergebnisses zwischenspeichern
	mov	r5, b	 
	mov	r3, a
	mov	a, SEED
	mov	b, #0x6f	; High-Byte des Multiplikators
	mul	ab
	add	a, r5		; Highbyte des Ergebnisses der ersten
	mov	r4, a		; Multiplikation addieren
	mov	a, r3
	add	a, #0xb9	
	mov	r3, a
	mov	a, r4
	addc	a, #0x7a
	mov	r4, a
;	mov	a, r3
	rlc	a
	mov	r5,#11

shift_loop:		; 16-Bit Shift der Register r4/r3
	mov	a, r3
	rlc	a
	mov	r3, a
	mov	a, r4
	rlc	a
	mov	r4, a
	djnz	r5, shift_loop

	mov	SEED+1, r4
	mov	a, r3
	mov	SEED, a
	ret
;}}}
rnd: ;{{{
	push	acc
	acall	rand
	pop	b
	div	ab
	mov	a, b
	ret
;}}}
;; Datas
clr_scr:
.byte	ESC,"[2J",0
buff_decimal:
.skip	3
.byte	0

.end
